#define __OCICPP_INTERNAL_USE_
#include "Connection.h"


Connection::Connection(OCIEnv *env,const string &tnsname,const string &user,const string &password):
                          envhp(env),srvhp(0),errhp(0),svchp(0),authp(0) 
{
	(void) OCIHandleAlloc( (dvoid *) envhp, (dvoid **) &errhp, OCI_HTYPE_ERROR, (size_t) 0, (dvoid **) 0);
	(void) OCIHandleAlloc( (dvoid *) envhp, (dvoid **) &srvhp, OCI_HTYPE_SERVER, (size_t) 0, (dvoid **) 0);
	(void) OCIHandleAlloc( (dvoid *) envhp, (dvoid **) &svchp, OCI_HTYPE_SVCCTX, (size_t) 0, (dvoid **) 0);
	CHECKERR(errhp,OCIServerAttach( srvhp, errhp, (text *)tnsname.c_str(), strlen(tnsname.c_str()), 0));
	(void) OCIAttrSet( (dvoid *) svchp, OCI_HTYPE_SVCCTX, (dvoid *)srvhp, (ub4) 0, OCI_ATTR_SERVER, (OCIError *) errhp);
	(void) OCIHandleAlloc((dvoid *) envhp, (dvoid **)&authp, (ub4) OCI_HTYPE_SESSION, (size_t) 0, (dvoid **) 0);
	(void) OCIAttrSet((dvoid *) authp, (ub4) OCI_HTYPE_SESSION, (dvoid *) user.c_str(), (ub4) strlen(user.c_str()), (ub4) OCI_ATTR_USERNAME, errhp);
	(void) OCIAttrSet((dvoid *) authp, (ub4) OCI_HTYPE_SESSION, (dvoid *) password.c_str(), (ub4) strlen(password.c_str()), (ub4) OCI_ATTR_PASSWORD, errhp);
	CHECKERR(errhp,OCISessionBegin ( svchp, errhp, authp, OCI_CRED_RDBMS, (ub4) OCI_DEFAULT));
	(void) OCIAttrSet((dvoid *) svchp, (ub4) OCI_HTYPE_SVCCTX, (dvoid *) authp, (ub4) 0, (ub4) OCI_ATTR_SESSION, errhp);
	(void) OCIHandleAlloc((dvoid *)envhp, (dvoid **)&trans, OCI_HTYPE_TRANS, 0, 0);
	(void) OCIAttrSet((dvoid *)svchp, OCI_HTYPE_SVCCTX, (dvoid *)trans, 0, OCI_ATTR_TRANS, errhp);
}

void Connection::init(OCIEnv *env,const string &tnsname,const string &user,const string &password) {
	envhp=env; srvhp=0; errhp=0; svchp=0; authp=0; 
	(void) OCIHandleAlloc( (dvoid *) envhp, (dvoid **) &errhp, OCI_HTYPE_ERROR, (size_t) 0, (dvoid **) 0);
	(void) OCIHandleAlloc( (dvoid *) envhp, (dvoid **) &srvhp, OCI_HTYPE_SERVER, (size_t) 0, (dvoid **) 0);
	(void) OCIHandleAlloc( (dvoid *) envhp, (dvoid **) &svchp, OCI_HTYPE_SVCCTX, (size_t) 0, (dvoid **) 0);
	CHECKERR(errhp,OCIServerAttach( srvhp, errhp, (text *)tnsname.c_str(), strlen(tnsname.c_str()), 0));
	(void) OCIAttrSet( (dvoid *) svchp, OCI_HTYPE_SVCCTX, (dvoid *)srvhp, (ub4) 0, OCI_ATTR_SERVER, (OCIError *) errhp);
	(void) OCIHandleAlloc((dvoid *) envhp, (dvoid **)&authp, (ub4) OCI_HTYPE_SESSION, (size_t) 0, (dvoid **) 0);
	(void) OCIAttrSet((dvoid *) authp, (ub4) OCI_HTYPE_SESSION, (dvoid *) user.c_str(), (ub4) strlen(user.c_str()), (ub4) OCI_ATTR_USERNAME, errhp);
	(void) OCIAttrSet((dvoid *) authp, (ub4) OCI_HTYPE_SESSION, (dvoid *) password.c_str(), (ub4) strlen(password.c_str()), (ub4) OCI_ATTR_PASSWORD, errhp);
	CHECKERR(errhp,OCISessionBegin ( svchp, errhp, authp, OCI_CRED_RDBMS, (ub4) OCI_DEFAULT));
	(void) OCIAttrSet((dvoid *) svchp, (ub4) OCI_HTYPE_SVCCTX, (dvoid *) authp, (ub4) 0, (ub4) OCI_ATTR_SESSION, errhp);
	(void) OCIHandleAlloc((dvoid *)envhp, (dvoid **)&trans, OCI_HTYPE_TRANS, 0, 0);
	(void) OCIAttrSet((dvoid *)svchp, OCI_HTYPE_SVCCTX, (dvoid *)trans, 0, OCI_ATTR_TRANS, errhp);
}

Connection::~Connection() {
	if (errhp) {
		CHECKERR(errhp,OCIServerDetach( srvhp, errhp, OCI_DEFAULT )); 
	}
	if (srvhp) {
		CHECKERR(errhp,OCIHandleFree((dvoid *) srvhp, OCI_HTYPE_SERVER));
	}
	if (svchp) (void) OCIHandleFree((dvoid *) svchp, OCI_HTYPE_SVCCTX);
	if (errhp) (void) OCIHandleFree((dvoid *) errhp, OCI_HTYPE_ERROR);  
}


Cursor *Connection::execQuery(const string &query) {
	DEBUG(DLEV_INFO,"execQuery(%s) executed\n",query.c_str());
	Cursor *cur=(Cursor *) 0;
	try {
		cur=new Cursor(envhp,svchp,errhp);
		DEBUG(DLEV_DEBUG,"Prepare stmt\n");
		if(cur->prepare(query)==OCI_STMT_SELECT) {
			DEBUG(DLEV_DEBUG,"Execute Select Stmt\n");
			cur->exec();
			DEBUG(DLEV_DEBUG,"Describe\n");
			cur->describe();
			DEBUG(DLEV_DEBUG,"Define\n");
			cur->define();
			DEBUG(DLEV_DEBUG,"Query Executed ok\n");
		} else { // It was non-select request does not supported etall yet 
			DEBUG(DLEV_DEBUG,"Execute Non-select stmt\n");
			cur->exec(); 
		}
	} catch(OraError *err) {
		delete cur;
		throw; /* tell to application error occured */
	}
	return cur;
}

void Connection::execQuery(const string &query,Cursor &cur) {
	DEBUG(DLEV_INFO,"execQuery(%s) executed\n",query.c_str());
	cur.init(envhp,svchp,errhp);
	DEBUG(DLEV_DEBUG,"Prepare stmt\n");
	if(cur.prepare(query)==OCI_STMT_SELECT) {
		DEBUG(DLEV_DEBUG,"Execute Select Stmt\n");
		cur.exec();
		DEBUG(DLEV_DEBUG,"Describe\n");
		cur.describe();
		DEBUG(DLEV_DEBUG,"Define\n");
		cur.define();
		DEBUG(DLEV_DEBUG,"Query Executed ok\n");
	} else { // It was non-select request does not supported etall yet 
		DEBUG(DLEV_DEBUG,"Execute Non-select stmt\n");
		cur.exec(); 
	}
}

void Connection::execUpdate(const string &request) {
	DEBUG(DLEV_INFO,"execQuery(%s) executed\n",request.c_str());
	Cursor *cur=(Cursor *) 0;
	try {
		cur=new Cursor(envhp,svchp,errhp);
		cur->prepare(request);
		cur->exec();
		delete cur;
	} catch (OraError *err) {
		delete cur;
		throw; /* Tell to application error occured */
	}
}

void Connection::transStart(int flags=SERIALIZABLE) {
	CHECKERR(errhp,OCITransStart(svchp,errhp,(uword) 0,flags)); /* Detaching is not supperted yet so timeout is ignored */
}

void Connection::transCommit() {
	CHECKERR(errhp,OCITransCommit(svchp,errhp,OCI_DEFAULT));
}
/* Hm... we should pass a dvoid * to Rollback and OCISvcCtx to Commit isn't it an architecture bug ? */
/* It was a bug in documentation $ORACLE_HOME/rdbms/demo/ociap.h contains this prototype of OCITransRollback() :
   sword   OCITransRollback  (OCISvcCtx *svchp, OCIError *errhp, ub4 flags);
and here's a documentation prototype:
   sword   OCITransRollback  (dvoid *svchp, OCIError *errhp, ub4 flags);
*/
void Connection::transRollback() {
	CHECKERR(errhp,OCITransRollback(svchp,errhp,OCI_DEFAULT));
}		