#define __OCICPP_INTERNAL_USE_
#include "Cursor.h"



Cursor::Cursor(OCIEnv *env,OCISvcCtx *svc,OCIError *err):envhp(env),svchp(svc),stmthp(0),paramd(0),
														errhp(err),defineArr(0),head(0),cell(0),
														nulltext(default_null_text)
{
	//Alloc statement handle
	status=OCIHandleAlloc( (dvoid *) envhp, (dvoid **) &stmthp, OCI_HTYPE_STMT, (size_t) 0, (dvoid **) 0);
	if(status!=OCI_SUCCESS) throw new OraError("Handle Alloc Failed File: "__FILE__,status);
}

void Cursor::init(OCIEnv *env,OCISvcCtx *svc,OCIError *err) {
	envhp=env;
	svchp=svc;
	stmthp=0;
	paramd=0;
	errhp=err;
	defineArr=0;
	head=0;
	cell=0;
	nulltext=default_null_text;
	//Alloc statement handle
	status=OCIHandleAlloc( (dvoid *) envhp, (dvoid **) &stmthp, OCI_HTYPE_STMT, (size_t) 0, (dvoid **) 0);
	if(status!=OCI_SUCCESS) throw new OraError("Handle Alloc Failed File: "__FILE__,status);
}

Cursor::~Cursor() {
	delete[] cell;
	delete[] head;
	if(defineArr) free(defineArr);
	CHECKERR(errhp,OCIHandleFree(stmthp,OCI_HTYPE_STMT));
}

sb4 Cursor::prepare(const string &sql) {
	CHECKERR(errhp,OCIStmtPrepare(stmthp, errhp,(text *) sql.c_str() ,(ub4) strlen(sql.c_str()), (ub4) OCI_NTV_SYNTAX, (ub4) OCI_DEFAULT));
	CHECKERR(errhp,OCIAttrGet(stmthp,OCI_HTYPE_STMT,(dvoid *) &stmtType,(ub4 *) 0,OCI_ATTR_STMT_TYPE,errhp));
	return stmtType;
}

void Cursor::exec() {
	if(stmtType==OCI_STMT_SELECT) {
		CHECKERR(errhp,OCIStmtExecute(svchp, stmthp,errhp,(ub4) 0,(ub4) 0, NULL, NULL, OCI_DEFAULT));
	} else {
		CHECKERR(errhp,OCIStmtExecute(svchp, stmthp,errhp,(ub4) 1,(ub4) 0, NULL, NULL, OCI_DEFAULT));
	}
}

void Cursor::describe() {
	CHECKERR(errhp,OCIAttrGet(stmthp,OCI_HTYPE_STMT,(dvoid *) &nCols,(ub4 *) 0,OCI_ATTR_PARAM_COUNT,errhp));
	head=new TableHead[nCols];
	cell=new RowCell[nCols];
	for(int col=0;col<nCols;col++) {
		ub2 colType;
		text *colName;
		ub4 colNameLen;
		ub2 colSize;
		ub1 prec; /* For NUMBERs only */
		CHECKERR(errhp,OCIParamGet(stmthp,OCI_HTYPE_STMT,errhp,(dvoid **) &paramd,col+1));
		CHECKERR(errhp,OCIAttrGet((dvoid *) paramd,OCI_DTYPE_PARAM,(dvoid *) &colType,(ub4 *) 0,(ub4) OCI_ATTR_DATA_TYPE,errhp));
		CHECKERR(errhp,OCIAttrGet((dvoid *) paramd,OCI_DTYPE_PARAM,(dvoid **) &colName,(ub4 *) &colNameLen,(ub4) OCI_ATTR_NAME,errhp));
		CHECKERR(errhp,OCIAttrGet((dvoid *) paramd,OCI_DTYPE_PARAM,(dvoid *) &colSize,(ub4 *) 0,(ub4) OCI_ATTR_DATA_SIZE,errhp));
		if(colType==SQLT_NUM) {
			/* scale is a length of string contain number */
			CHECKERR(errhp,OCIAttrGet((dvoid *) paramd,OCI_DTYPE_PARAM,(dvoid *) &prec, (ub4 *) 0,(ub4) OCI_ATTR_PRECISION,errhp));
			DEBUG(DLEV_DEBUG,"Precision was: %du\n",(unsigned)prec);
			colSize=(ub2)prec;
		}
		head[col].setAttrType(colType);
		head[col].setAttrName(colName,colNameLen);
		head[col].setAttrSize(colSize);
	}
	if(nCols) haveResultSet=1;
	hashHead();
}
void Cursor::hashHead() {
	for(int col=0;col<nCols;col++) {
		string attrName;
		getColName(col,attrName);
		cols_map[attrName]=col;
		DEBUG(DLEV_DEBUG,"%s->%d\n",attrName.c_str(),cols_map[attrName]);
	}
}


void Cursor::define() {
	defineArr=(OCIDefine **)calloc(nCols,sizeof(OCIDefine *));
	correctTypes(); /* This is to correct e.g. NUMBER->STRING and over types what are not really usefull in host language */
	for(int i=0;i<nCols;i++) {
		cell[i].data=(dvoid *)calloc(head[i].getAttrSize()+1,sizeof(char));
		CHECKERR(errhp, OCIDefineByPos(stmthp, &defineArr[i], errhp, i+1, cell[i].data, (sword)head[i].getAttrSize(),head[i].getReqType(),
		         (dvoid *) &(cell[i].isNull), (ub2 *)&(cell[i].len), (ub2 *)0, OCI_DEFAULT));
	}	
	haveResultSet=1;
}	



int Cursor::fetch() {
	canFetch=0;
	if(haveResultSet) {
		status=OCIStmtFetch(stmthp,errhp,(ub4) 1,OCI_FETCH_NEXT,OCI_DEFAULT); 
		if(status==OCI_NO_DATA) {
			canFetch=0;
		} else if(status==OCI_SUCCESS || status==OCI_SUCCESS_WITH_INFO) {
			canFetch=1;
		} else throw new OraError(errhp,status);
	} 
	return canFetch;
}

/* 	get<Type> functions */
void Cursor::getStr(int col,string &str) const {
	if(!canFetch || col<0 || col>=nCols) throw new OraError(CELL_NOT_EXISTS);
	if(col<0 || col>=nCols) throw new OraError(CELL_NOT_EXISTS);
	if(cell[col].isNull==-1) {
		str.assign(nulltext);
		return;
	}
	switch(head[col].getReqType()) {
		default:
			throw new OraError(CELL_TYPE_UNSUPPORTED);
			break;
		case SQLT_STR:
			str.assign((char *)cell[col].data,cell[col].len);
			break;
		case SQLT_AVC:
			str.assign((char *)cell[col].data);
			break;
		case SQLT_LNG:
			str.assign((char *)cell[col].data);
			break;
		case SQLT_DAT:
			char *dat;
			char *month[]={"JAN","FEB","MAR","APR","MAY","JUN","JUL","AUG","SEP","OCT","NOV","DEC"};
			unsigned char cc,yy,mm,dd,hh,mi,ss;
			dat=(char *)calloc(8,1);
			cc=((unsigned char *)(cell[col].data))[0]; /* Century */
			yy=((unsigned char *)(cell[col].data))[1]; /* Year */
			mm=((char *)(cell[col].data))[2]; /* Month */
			dd=((char *)(cell[col].data))[3]; /* Day */
			hh=((char *)(cell[col].data))[4]; /* Hour */
			mi=((char *)(cell[col].data))[5]; /* Minute */
			ss=((char *)(cell[col].data))[6]; /* Seconds */
			cc=(unsigned char)abs(cc-100);
			yy=(unsigned char)abs(yy-100);
			sprintf(dat,"%d-%s-%d",dd,month[mm-1],(unsigned int)cc*100+(unsigned int)yy);
			str.assign(dat);
			free(dat);
			break;
	}
}

void Cursor::getStr(const string &colName,string &val) const {
	map<string,int>::const_iterator iter=cols_map.find(colName);
	if(iter!=cols_map.end()) {
		getStr(iter->second,val);
	} else {
		throw new OraError(CELL_NOT_EXISTS);
	}
}


void Cursor::getInt(int col,int &val) const {
	if(!canFetch) throw new OraError(CELL_NOT_EXISTS);
	if(cell[col].isNull==-1) val=0;
	else if(head[col].getReqType()==SQLT_STR || head[col].getReqType()==SQLT_LNG) {
		val=atoi((char *)cell[col].data);
	} else if(head[col].getReqType()==SQLT_DAT) {
		throw(new OraError("Error - Cannot convert date to int\n",-1));
	} else {
		throw new OraError(CELL_TYPE_UNSUPPORTED);
	}
}

void Cursor::getInt(const string &colName,int &val) const {
	map<string,int>::const_iterator iter=cols_map.find(colName);
	if(iter!=cols_map.end()) {
		getInt(iter->second,val);
	} else {
		throw new OraError(CELL_NOT_EXISTS);
	}
}

void Cursor::getDouble(int col,double &val) const {
	if(!canFetch) throw new OraError(CELL_NOT_EXISTS);
	if(cell[col].isNull==-1) val=0;
	else if(head[col].getReqType()==SQLT_STR || head[col].getReqType()==SQLT_LNG) {
		val=atof((char *)cell[col].data);
	} else if(head[col].getReqType()==SQLT_DAT) {
		throw(new OraError("Error - Cannot convert date to double\n",-1));
	} else {
		throw new OraError(CELL_TYPE_UNSUPPORTED);
	}
}

void Cursor::getDouble(const string &colName,double &val) const {
	map<string,int>::const_iterator iter=cols_map.find(colName);
	if(iter!=cols_map.end()) {
		getDouble(iter->second,val);
	} else {
		throw new OraError(CELL_NOT_EXISTS);
	}
}
/* ----------------------*/


int Cursor::getNCols() const {
	if(haveResultSet) return (int)nCols;
	throw new OraError("OCICPPLIB: Can't return number of columns: No result set for this cursor",OCICPPERROR);
}	

void Cursor::getColName(int col,string &ret_col) const {
	if(!haveResultSet) throw new OraError("OCICPPLIB: Cannot get column name: No result set for this cursor",OCICPPERROR);
	if(col<0 || col>=nCols) throw new OraError("OCICPPLIB: Cannot get column name: column number is out of range",OCICPPERROR);
	ret_col.assign((char *)head[col].getAttrName());
}

int Cursor::getColSize(int col) const {
	if(!haveResultSet) throw new OraError("OCICPPLIB: Cannot get column name: No result set for this cursor",OCICPPERROR);
	if(col<0 || col>=nCols) throw new OraError("OCICPPLIB: Cannot get column name: column number is out of range",OCICPPERROR);
	return (int)head[col].getAttrSize();
}

/* Set's text which will be returned on request of null cell default "" (empty text) */	
void Cursor::setNullText(const string &nulltext) {
	this->nulltext.assign(nulltext);
}

void Cursor::correctTypes() {
	for(int i=0;i<nCols;i++) {
		if(head[i].getAttrType()==SQLT_NUM) {
			head[i].setReqType(SQLT_STR);
			if(!head[i].getAttrSize()) { /* Precision is 0 choose our own len */
				head[i].setAttrSize(255); /* hope it's enough to keep most of oracle numbers */
			} else { 
				head[i].setAttrSize(head[i].getAttrSize()+1);
			}
		} else if(head[i].getAttrType()==SQLT_LNG) {
			head[i].setAttrSize(4000);
			head[i].setReqType(SQLT_STR);
		} else if(head[i].getAttrType()==SQLT_INT) {
			head[i].setAttrSize(12); /* Just to set host language int size on [LW]Intelx86 arcs should be 4 */
			head[i].setReqType(SQLT_STR);
		} else if(head[i].getAttrType()==SQLT_AFC) {
			head[i].setReqType(SQLT_AVC);
			head[i].setAttrSize(head[i].getAttrSize()+1);
		} else if(head[i].getAttrType()==SQLT_CHR) {
			head[i].setReqType(SQLT_STR);
			head[i].setAttrSize(head[i].getAttrSize()+1);
		} else {
			head[i].setReqType(head[i].getAttrType());
		}
	} 		
}